﻿using System;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace Train
{
    /// <summary>
    /// Interaction logic for FormLight.xaml
    /// </summary>
    
    public partial class FormLight : Window
    {
        private int currentIndex = 1;
        private Data daten;
        private ToggleButton[] toggleButton = new ToggleButton[12];
        
        public FormLight()
        {
            InitializeComponent();
        }
        
        public void ShowFormAndInput(Data Daten)
        {
            daten = Daten;
            ShowDialog();
        }

        private void ShowButton(int StartIndex, int EndIndex)
        {
            Wrap.Children.Clear();
            for (int i = 0; i < (EndIndex - StartIndex + 1); i++)
            {
                // Toogle Button
                toggleButton[i] = new ToggleButton();
                toggleButton[i].Width = 245;
                toggleButton[i].Height = 75;
                toggleButton[i].Margin = new Thickness(5);
                toggleButton[i].Tag = StartIndex + i - 1; // Index zuweisen

                // Grid
                Grid grid = new Grid();
                grid.Width = 245;
                grid.Height = 75;
                grid.ColumnDefinitions.Add(new ColumnDefinition());
                grid.ColumnDefinitions.Add(new ColumnDefinition());

                // Grid als Content des ToogleButton
                toggleButton[i].Content = grid;
                
                // Textblock
                TextBlock textBlock = new TextBlock();
                textBlock.FontSize = 16;
                textBlock.FontWeight = FontWeights.Bold;
                textBlock.Text = daten.LigthData[StartIndex + i - 1].Headline; ;

                // Textblock innerhalb des Grid Plazieren
                Grid.SetColumn(textBlock, 0);

                // Textblock zum Grid zuweisen
                grid.Children.Add(textBlock);

                // Image
                Image image = new Image();
                image.Margin = new Thickness(10,0,0,0);
                try
                {
                    string dir = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName) + "\\icons\\";
                    string fileName = daten.LigthData[StartIndex + i - 1].IconFileName;
                    string pfad = dir + fileName;
                    image.Source = BitmapFrame.Create(new Uri(pfad));
                }
                catch
                {
                }
                
                // Image innerhalb des Grid platzieren
                Grid.SetColumn(image, 1);
                // Image zum Grid zuweisen
                grid.Children.Add(image);
                
                // ToggleButton als Content zum WrapPanel
                Wrap.Children.Add(toggleButton[i]);
                
                // Ereignisse für den ToggleButton
                toggleButton[i].Click += new RoutedEventHandler(toggleButton_Click);
                toggleButton[i].Checked += new RoutedEventHandler(toggleButton_Checked);
                toggleButton[i].Unchecked += new RoutedEventHandler(toggleButton_Unchecked);
                toggleButton[i].IsChecked = daten.LigthData[StartIndex + i - 1].Status;
            }
        }

        void toggleButton_Unchecked(object sender, RoutedEventArgs e)
        {
            ToggleButton tb = (ToggleButton)sender;
            ((Grid)tb.Content).Background = new SolidColorBrush(Colors.Gainsboro);
        }

        void toggleButton_Checked(object sender, RoutedEventArgs e)
        {
            ToggleButton tb = (ToggleButton)sender;
            ((Grid)tb.Content).Background = new SolidColorBrush(Colors.Yellow);
            
        }

        void toggleButton_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton checkbox = (ToggleButton)sender;
            int index = (int)checkbox.Tag;
            daten.LigthData[index].Status = !daten.LigthData[index].Status;
            ClickCheckBox(index, true);
        }

        private void ClickCheckBox(int Index, bool CheckALL)
        {
            int indexCheckbox = Index;
            while (indexCheckbox > 11)
            {
                indexCheckbox = indexCheckbox - 12;
            }
            toggleButton[indexCheckbox].IsChecked = daten.LigthData[Index].Status;
            int adress = daten.LigthData[Index].Adress;
            if (daten.LigthData[Index].Chance == false)
            {
                CommunicationToInterface.Ligth(adress, (bool) toggleButton[indexCheckbox].IsChecked);
            }
            else
            {
                CommunicationToInterface.Ligth(adress, ! (bool) toggleButton[indexCheckbox].IsChecked);
            }

            // Prüfen, ob alle Lichter der Modellbahn angeschaltet bzw. ausgeschaltet sind
            if (CheckALL == true)
            {
                bool allOn = true;
                for (int i = 0; i < daten.LigthData.Count; i++)
                {
                    if (daten.LigthData[i].Status == false)
                    {
                        allOn = false;
                        break;
                    }
                }
                if (allOn == true) toggleButtonAllOn.IsChecked = true;
                else toggleButtonAllOn.IsChecked = false;
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            int number;
            if (daten.LigthData.Count >= 12)
            {
                number = 12;
                if (daten.LigthData.Count > 12) buttonPre.IsEnabled = true;
            }
            else
            {
                number = daten.LigthData.Count;
                buttonPre.IsEnabled = false;
                buttonBack.IsEnabled = false;
            }
            ShowButton(1, number);
        }

        private void buttonPre_Click(object sender, RoutedEventArgs e)
        {
            currentIndex = currentIndex + 12;
            int endIndex = currentIndex + 11;
            if (endIndex > daten.LigthData.Count) endIndex = daten.LigthData.Count;
            buttonBack.IsEnabled = true;
            if (endIndex <= daten.LigthData.Count)
            {
                buttonPre.IsEnabled = false;
            }
            ShowButton(currentIndex, endIndex);
        }

        private void buttonBack_Click(object sender, RoutedEventArgs e)
        {
            currentIndex = currentIndex - 12;
            int endIndex = currentIndex + 11;
            buttonPre.IsEnabled = true;
            if (currentIndex <= 1)
            {
                buttonBack.IsEnabled = false;
            }
            ShowButton(currentIndex, endIndex);
        }

        private void toggleButtonAllOn_Click(object sender, RoutedEventArgs e)
        {
            // alle Lampen an bzw. ausschalten
            for (int i = 0; i < daten.LigthData.Count; i++)  
            {
                daten.LigthData[i].Status = (bool) toggleButtonAllOn.IsChecked;
                ClickCheckBox(i, false);
                DoEvents();
            }
        }

        private static void DoEvents()
        {
            if (Application.Current != null)
                Application.Current.Dispatcher.Invoke(DispatcherPriority.Background, new ThreadStart(delegate { }));
        }
       
    }
}
